'use strict';

var _inherits = require('babel-runtime/helpers/inherits')['default'];

var _classCallCheck = require('babel-runtime/helpers/class-call-check')['default'];

var _Object$assign = require('babel-runtime/core-js/object/assign')['default'];

var _interopRequireDefault = require('babel-runtime/helpers/interop-require-default')['default'];

exports.__esModule = true;

var _visitor = require('./visitor');

var _visitor2 = _interopRequireDefault(_visitor);

var _translateUtil = require('./translate-util');

/**
 * Transforms source code, translating and inlining `formatMessage` calls
 **/

var Extractor = (function (_Visitor) {
  function Extractor(options) {
    _classCallCheck(this, Extractor);

    _Visitor.call(this, options);
    this.patterns = null;
  }

  _inherits(Extractor, _Visitor);

  Extractor.prototype.extract = function extract(_ref) {
    var sourceCode = _ref.sourceCode;
    var sourceFileName = _ref.sourceFileName;

    this.patterns = {};
    this.run({ sourceCode: sourceCode, sourceFileName: sourceFileName });
    return this.patterns;
  };

  Extractor.prototype.exitFormatCall = function exitFormatCall(node) {
    this.savePattern(node);
  };

  Extractor.prototype.exitTranslateCall = function exitTranslateCall(node) {
    this.savePattern(node);
  };

  Extractor.prototype.savePattern = function savePattern(node) {
    if (!this.isReplaceable(node)) return;

    var pattern = this.getStringValue(node.arguments[0]);
    var error = this.getPatternError(pattern);
    if (error) {
      this.reportError(node, 'SyntaxError: pattern is invalid', error.message);
    } else {
      var key = this.getKey(pattern);
      this.patterns[key] = _translateUtil.getKeyNormalized(pattern);
    }
  };

  Extractor.extract = function extract(source, options) {
    return new Extractor(options).extract(source);
  };

  Extractor.extractFromFiles = function extractFromFiles(files, options) {
    var _JSON$stringify;

    var extractor = new Extractor(options);
    var translations = {};
    extractor.forEachFile(files, function (source) {
      var patterns = extractor.extract(source);
      _Object$assign(translations, patterns);
    });
    var json = JSON.stringify((_JSON$stringify = {}, _JSON$stringify[extractor.locale] = translations, _JSON$stringify), null, '  ');
    if (options.outFile) {
      extractor.emitFile(options.outFile, json);
    } else {
      extractor.emitResult(json);
    }
  };

  return Extractor;
})(_visitor2['default']);

exports['default'] = Extractor;
module.exports = exports['default'];