'use strict';

var _inherits = require('babel-runtime/helpers/inherits')['default'];

var _classCallCheck = require('babel-runtime/helpers/class-call-check')['default'];

var _interopRequireDefault = require('babel-runtime/helpers/interop-require-default')['default'];

var _interopRequireWildcard = require('babel-runtime/helpers/interop-require-wildcard')['default'];

exports.__esModule = true;

var _visitor = require('./visitor');

var _visitor2 = _interopRequireDefault(_visitor);

var _path = require('path');

var _babelCore = require('babel-core');

var babel = _interopRequireWildcard(_babelCore);

var _sourceMap = require('source-map');

var _sourceMap2 = _interopRequireDefault(_sourceMap);

var _messageFormatParser = require('message-format/parser');

var _messageFormatParser2 = _interopRequireDefault(_messageFormatParser);

var _transpiler = require('./transpiler');

var _transpiler2 = _interopRequireDefault(_transpiler);

var types = babel.types;

/**
 * Transforms source code, translating and inlining `formatMessage` calls
 **/

var Inliner = (function (_Visitor) {
  function Inliner(options) {
    _classCallCheck(this, Inliner);

    _Visitor.call(this, options);
    this.declarations = [];
  }

  _inherits(Inliner, _Visitor);

  Inliner.prototype.inline = function inline(_ref) {
    var sourceCode = _ref.sourceCode;
    var sourceFileName = _ref.sourceFileName;
    var sourceMapName = _ref.sourceMapName;
    var inputSourceMap = _ref.inputSourceMap;

    this.declarations.length = 0;
    return this.run({ sourceCode: sourceCode, sourceFileName: sourceFileName, sourceMapName: sourceMapName, inputSourceMap: inputSourceMap });
  };

  Inliner.prototype.exitFormatCall = function exitFormatCall(node, parent, scope, traverser) {
    if (!this.isReplaceable(node)) return;

    var originalPattern = this.getStringValue(node.arguments[0]);
    var localeArg = node.arguments[2];
    var locale = localeArg && this.getStringValue(localeArg) || this.locale;

    var pattern = this.getTranslation(originalPattern, locale, node);
    var patternAst = _messageFormatParser2['default'].parse(pattern);

    var _Transpiler$transpile = _transpiler2['default'].transpile(patternAst, {
      originalPattern: originalPattern, locale: locale, node: node, traverser: traverser
    });

    var replacement = _Transpiler$transpile.replacement;
    var declaration = _Transpiler$transpile.declaration;

    this.addDeclaration(declaration);
    traverser.replaceWith(replacement);
  };

  Inliner.prototype.exitTranslateCall = function exitTranslateCall(node, parent, scope, traverser) {
    if (!this.isReplaceable(node)) return;

    var originalPattern = this.getStringValue(node.arguments[0]);
    var localeArg = node.arguments[1];
    var locale = localeArg && this.getStringValue(localeArg) || this.locale;

    var pattern = this.getTranslation(originalPattern, locale, node);
    return traverser.replaceWith(types.literal(pattern));
  };

  Inliner.prototype.exitProgram = function exitProgram(node) {
    node.body = node.body.concat(this.declarations);
    this.declarations.length = 0;
  };

  Inliner.prototype.shouldOutputCode = function shouldOutputCode() {
    return true;
  };

  Inliner.prototype.getTranslation = function getTranslation(originalPattern, locale, node) {
    var pattern = this.translate(originalPattern, locale);
    if (pattern != null) {
      return pattern;
    }

    var replacement = this.missingReplacement || originalPattern;
    var message = 'no ' + locale + ' translation found for key ' + JSON.stringify(this.getKey(originalPattern));

    if (this.missingTranslation === 'ignore') {} else if (this.missingTranslation === 'warning') {
      this.reportWarning(node, 'Warning: ' + message);
    } else {
      // 'error'
      this.reportError(node, 'Error: ' + message);
      throw new Error(message);
    }

    return replacement;
  };

  Inliner.prototype.addDeclaration = function addDeclaration(declaration) {
    if (declaration) {
      var found = this.declarations.some(function (_ref2) {
        var id = _ref2.id;
        return id === declaration.id;
      });
      if (!found) {
        this.declarations.push(declaration);
      }
    }
  };

  Inliner.prototype.sourceMapComment = function sourceMapComment(path) {
    return '\n//# sourceMappingURL=' + path;
  };

  Inliner.prototype.sourceMapInlineComment = function sourceMapInlineComment(map) {
    return this.sourceMapComment('data:application/json;base64,' + new Buffer(JSON.stringify(map)).toString('base64'));
  };

  Inliner.inline = function inline(source, options) {
    return new Inliner(options).inline(source);
  };

  Inliner.inlineFiles = function inlineFiles(files, options) {
    if (options.outDir) {
      Inliner.inlineManyToMany(files, options);
    } else {
      Inliner.inlineManyToOne(files, options);
    }
  };

  Inliner.inlineManyToMany = function inlineManyToMany(files, options) {
    var inliner = new Inliner(options);
    var root = _path.resolve(options.root);
    var outDir = options.outDir.replace(/\/$/, '') + '/';
    inliner.forEachFile(files, function (source) {
      var result = inliner.inline(source);
      var outFileName = _path.join(outDir, _path.relative(root, source.sourceFileName));

      if (options.sourceMaps === 'inline' && result.map) {
        result.code += inliner.sourceMapInlineComment(result.map);
      } else if (options.sourceMaps && result.map) {
        var mapFileName = outFileName + '.map';
        inliner.emitFile(mapFileName, JSON.stringify(result.map));
        result.code += inliner.sourceMapComment(_path.basename(mapFileName));
      }

      inliner.emitFile(outFileName, result.code, result.map);
    });
  };

  Inliner.inlineManyToOne = function inlineManyToOne(files, options) {
    var inliner = new Inliner(options);
    var outFileName = options.outFile || options.filename;
    var map = new _sourceMap2['default'].SourceMapGenerator({
      file: outFileName
    });
    var code = '';
    var offset = 0;

    inliner.forEachFile(files, function (source) {
      var result = inliner.inline(source);
      var filename = source.souceFileName;
      if (result.map) {
        var consumer = new _sourceMap2['default'].SourceMapConsumer(result.map);
        map._sources.add(filename);
        map.setSourceContent(filename, source.sourceCode);
        consumer.eachMapping(function (mapping) {
          map._mappings.add({
            generatedLine: mapping.generatedLine + offset,
            generatedColumn: mapping.generatedColumn,
            originalLine: mapping.originalLine,
            originalColumn: mapping.originalColumn,
            source: filename
          });
        });
      }

      code += result.code + '\n';
      offset = code.split('\n').length;
    });

    if (options.sourceMaps === 'inline') {
      code += inliner.sourceMapInlineComment(map);
    } else if (options.sourceMaps) {
      var mapFileName = outFileName + '.map';
      inliner.emitFile(mapFileName, JSON.stringify(map));
      code += inliner.sourceMapComment(_path.basename(mapFileName));
    }

    if (options.outFile) {
      inliner.emitFile(options.outFile, code, map);
    } else {
      inliner.emitResult(code);
    }
  };

  return Inliner;
})(_visitor2['default']);

exports['default'] = Inliner;
module.exports = exports['default'];

// do nothing