'use strict';

var _getIterator = require('babel-runtime/core-js/get-iterator')['default'];

var _interopRequireDefault = require('babel-runtime/helpers/interop-require-default')['default'];

exports.__esModule = true;
exports.getTranslate = getTranslate;
exports.getGetKey = getGetKey;
exports.getKeyNormalized = getKeyNormalized;
exports.getKeyUnderscored = getKeyUnderscored;
exports.getKeyUnderscoredCrc32 = getKeyUnderscoredCrc32;

var _speakingurl = require('speakingurl');

var _speakingurl2 = _interopRequireDefault(_speakingurl);

var _crc32 = require('crc32');

var _crc322 = _interopRequireDefault(_crc32);

var _messageFormatParser = require('message-format/parser');

var _messageFormatParser2 = _interopRequireDefault(_messageFormatParser);

var _messageFormatPrinter = require('message-format/printer');

var _messageFormatPrinter2 = _interopRequireDefault(_messageFormatPrinter);

function getTranslate(options) {
  if (options.translate) {
    return options.translate;
  }

  if (!options.translations) {
    return function (pattern) {
      return pattern;
    };
  }

  var getKey = getGetKey(options);
  return function (pattern, locales) {
    var translations = options.translations;
    var key = getKey(pattern);
    locales = [].concat(locales);
    for (var _iterator = locales, _isArray = Array.isArray(_iterator), _i = 0, _iterator = _isArray ? _iterator : _getIterator(_iterator);;) {
      var _ref;

      if (_isArray) {
        if (_i >= _iterator.length) break;
        _ref = _iterator[_i++];
      } else {
        _i = _iterator.next();
        if (_i.done) break;
        _ref = _i.value;
      }

      var locale = _ref;

      var parts = locale.split('-');
      while (parts.length) {
        var current = parts.join('-');
        if (current in translations && key in translations[current]) {
          return translations[current][key];
        }
        parts.pop();
      }
    }
  };
}

function getGetKey(options) {
  switch (options.keyType) {
    case 'underscored_crc32':
      return getKeyUnderscoredCrc32;
    case 'underscored':
      return getKeyUnderscored;
    case 'normalized':
      return getKeyNormalized;
    default:
      // 'literal'
      return function (pattern) {
        return pattern;
      };
  }
}

function getKeyNormalized(pattern) {
  return _messageFormatPrinter2['default'].print(_messageFormatParser2['default'].parse(pattern)).replace(/\s+/g, ' ');
}

function getKeyUnderscored(pattern) {
  pattern = getKeyNormalized(pattern);
  return _speakingurl2['default'](pattern, { separator: '_', lang: false }).replace(/[-_]+/g, '_').slice(0, 50);
}

function getKeyUnderscoredCrc32(pattern) {
  pattern = getKeyNormalized(pattern);
  var underscored = _speakingurl2['default'](pattern, { separator: '_', lang: false }).replace(/[-_]+/g, '_').slice(0, 50);
  var crc = _crc322['default'](pattern.length + ':' + pattern).toString(16);
  return underscored + '_' + crc;
}