'use strict';

var _classCallCheck = require('babel-runtime/helpers/class-call-check')['default'];

var _Object$keys = require('babel-runtime/core-js/object/keys')['default'];

var _interopRequireDefault = require('babel-runtime/helpers/interop-require-default')['default'];

var _interopRequireWildcard = require('babel-runtime/helpers/interop-require-wildcard')['default'];

exports.__esModule = true;

var _localesJson = require('./locales.json');

var _localesJson2 = _interopRequireDefault(_localesJson);

var _messageFormatLibLookupClosestLocale = require('message-format/lib/lookup-closest-locale');

var _messageFormatLibLookupClosestLocale2 = _interopRequireDefault(_messageFormatLibLookupClosestLocale);

var _messageFormatLibData = require('message-format/lib/data');

var _translateUtil = require('./translate-util');

var _babelCore = require('babel-core');

var babel = _interopRequireWildcard(_babelCore);

var types = babel.types;
var pluralVars = {
  i: babel.parse('i=' + _localesJson2['default'].pluralVars.i).body[0].expression,
  v: babel.parse('v=' + _localesJson2['default'].pluralVars.v).body[0].expression,
  w: babel.parse('w=' + _localesJson2['default'].pluralVars.w).body[0].expression,
  f: babel.parse('f=' + _localesJson2['default'].pluralVars.f).body[0].expression,
  t: babel.parse('t=' + _localesJson2['default'].pluralVars.t).body[0].expression
};

/**
 * Transpiler
 *
 * Turns this:
 *  [ "You have ", [ "numBananas", "plural", 0, {
 *       "=0": [ "no bananas" ],
 *      "one": [ "a banana" ],
 *    "other": [ [ '#' ], " bananas" ]
 *  } ], " for sale." ]
 *
 * into ast of this:
 *  `(function(args, locale) {
 *    return "You have " + formatMessage.plural(locale, args["numBananas"], 0, {
 *      "=0": "no bananas",
 *      "one": "a banana",
 *      "other": function() {
 *        return args["numBananas"] + " bananas";
 *      }
 *    }) + " for sale.";
 *  })(args, "en")`
 **/

var Transpiler = (function () {
  function Transpiler() {
    var options = arguments[0] === undefined ? {} : arguments[0];

    _classCallCheck(this, Transpiler);

    this.vars = {};
    this.originalPattern = options.originalPattern;
    this.locale = options.locale || 'en';
    this.node = options.node;
  }

  Transpiler.prototype.countReferences = function countReferences(elements) {
    var references = {};

    function count(elements) {
      elements.forEach(function (element) {
        if (!Array.isArray(element)) return;
        var name = element[0];
        references[name] = references[name] || 0;
        ++references[name];
      });
    }

    count(elements);
    return references;
  };

  Transpiler.prototype.checkUseArgExpressions = function checkUseArgExpressions(elements) {
    var _this = this;

    this.useArgExpressions = false;
    var blacklist = ['plural', 'selectordinal', 'select'];
    var noIntermediateVars = elements.every(function (element) {
      return typeof element === 'string' || blacklist.indexOf(element[1]) === -1;
    });
    if (!noIntermediateVars) return;

    var references = {};
    elements.forEach(function (element) {
      if (Array.isArray(element)) {
        var _name = element[0];
        references[_name] = references[_name] || 0;
        ++references[_name];
      }
      // plural, selectordinal, and select already disallowed no need to recurse
    });

    var paramsNode = this.node.arguments[1];
    var noPossibleSideEffects = paramsNode && (types.isObjectExpression(paramsNode) && paramsNode.properties.every(function (prop) {
      return types.isIdentifier(prop.key) && (types.isIdentifier(prop.value) || types.isLiteral(prop.value) || references[prop.key.name] === 1);
    }));
    if (noPossibleSideEffects) {
      this.useArgExpressions = {};
      paramsNode.properties.forEach(function (prop) {
        _this.useArgExpressions[prop.key.name] = prop.value;
      });
    }
  };

  Transpiler.prototype.transpile = function transpile(elements) {
    var _this2 = this;

    this.vars = {};

    if (elements.length === 0) {
      return { replacement: types.literal('') };
    }

    if (elements.length === 1 && typeof elements[0] === 'string') {
      return { replacement: types.literal(elements[0]) };
    }

    this.checkUseArgExpressions(elements);

    var concatElements = elements.map(function (element) {
      return _this2.transpileElement(element, null);
    }).reduce(function (left, right) {
      return types.binaryExpression('+', left, right);
    });

    if (this.useArgExpressions) {
      return { replacement: concatElements };
    }

    var key = this.originalPattern;
    var functionName = '$$_' + _translateUtil.getKeyUnderscoredCrc32(key);
    var replacement = types.callExpression(types.identifier(functionName), // call our declared function
    // pass in original function and arguments
    [this.node.callee, this.node.arguments[1]]);

    var body = [types.returnStatement(concatElements)];
    if (_Object$keys(this.vars).length) {
      body.unshift(types.variableDeclaration('var', // type
      _Object$keys(this.vars).map(function (name) {
        return types.variableDeclarator(types.identifier(name), _this2.vars[name]);
      })));
    }
    var declaration = types.functionDeclaration(types.identifier(functionName), [this.node.callee, types.identifier('args')], types.blockStatement(body), false, // isGenerator
    false // isExpression
    );

    return { replacement: replacement, declaration: declaration };
  };

  Transpiler.prototype.transpileSub = function transpileSub(elements, parent) {
    var _this3 = this;

    if (elements.length === 0) {
      return types.literal('');
    }

    if (elements.length === 1 && typeof elements[0] === 'string') {
      return types.literal(elements[0]);
    }

    return elements.map(function (element) {
      return _this3.transpileElement(element, parent);
    }).reduce(function (left, right) {
      return types.binaryExpression('+', left, right);
    });
  };

  Transpiler.prototype.transpileElement = function transpileElement(element, parent) {
    if (typeof element === 'string') {
      return types.literal(element);
    }

    var id = element[0];
    var type = element[1];
    var style = element[2];

    if (id === '#') {
      id = parent[0];
      var _offset = parent[2];
      return this.transpileNumber(id, _offset, null);
    }

    switch (type) {
      case 'number':
      case 'ordinal': // TODO: rbnf
      case 'spellout': // TODO: rbnf
      case 'duration':
        // TODO: duration
        return this.transpileNumber(id, 0, style);
      case 'date':
      case 'time':
        return this.transpileDateTime(id, type, style);
      case 'plural':
      case 'selectordinal':
        var offset = element[2];
        var options = element[3];
        return this.transpilePlural(id, type, offset, options);
      case 'select':
        return this.transpileSelect(id, style);
      default:
        return this.transpileArgument(id);
    }
  };

  Transpiler.prototype.transpileNumber = function transpileNumber(id, offset, style) {
    var value = this.transpileArgument(id);
    if (offset) {
      value = types.binaryExpression('-', value, types.literal(offset));
    }
    var args = [types.literal(this.locale), value];
    if (style) {
      args.push(types.literal(style));
    }
    return types.callExpression(types.memberExpression(this.node.callee, types.identifier('number')), args);
  };

  Transpiler.prototype.transpileDateTime = function transpileDateTime(id, type, style) {
    var args = [types.literal(this.locale), this.transpileArgument(id)];
    if (style) {
      args.push(types.literal(style));
    }
    return types.callExpression(types.memberExpression(this.node.callee, types.identifier(type)), args);
  };

  Transpiler.prototype.transpilePlural = function transpilePlural(id, type, offset, children) {
    var _this4 = this;

    var parent = [id, type, offset /*, children*/];
    var closest = _messageFormatLibLookupClosestLocale2['default'](this.locale, _localesJson2['default'].locales);
    var conditions = _localesJson2['default'].locales[closest].plurals[type === 'selectordinal' ? 'ordinal' : 'cardinal'];
    var s = types.identifier('s');
    var n = types.identifier('n');
    var other = types.literal('');
    var vars = [types.assignmentExpression('=', s, this.transpileArgument(id)), types.assignmentExpression('=', n, types.unaryExpression('+', s, true /* isPrefix */))];
    var pvars = [];
    var refsI = false;
    var refsV = false;
    var refsW = false;
    var refsF = false;
    var refsT = false;
    if (offset) {
      pvars.push(types.assignmentExpression('=', n, types.assignmentExpression('=', s, types.binaryExpression('-', types.unaryExpression('+', s, true /* isPrefix */), types.literal(offset)))));
    }

    var exactConditions = [];
    var keyConditions = [];
    _Object$keys(children).forEach(function (key) {
      var expr = _this4.transpileSub(children[key], parent);
      if (key === 'other') {
        other = expr;
      } else if (key.charAt(0) === '=') {
        var test = types.binaryExpression('===', n, types.literal(+key.slice(1)));
        exactConditions.push({ test: test, expr: expr });
      } else if (key in conditions) {
        var cond = conditions[key];
        if (/\bi\b/.test(cond)) {
          refsI = !(_this4.vars.i = null);
        }
        if (/\bv\b/.test(cond)) {
          refsV = !(_this4.vars.v = null);
        }
        if (/\bw\b/.test(cond)) {
          refsW = !(_this4.vars.w = null);
        }
        if (/\bf\b/.test(cond)) {
          refsF = !(_this4.vars.f = null);
        }
        if (/\bt\b/.test(cond)) {
          refsT = !(_this4.vars.t = null);
        }
        var test = babel.parse(cond).body[0].expression;
        keyConditions.push({ test: test, expr: expr });
      }
    });
    if (refsI) {
      pvars.push(pluralVars.i);
    }
    if (refsV) {
      pvars.push(pluralVars.v);
    }
    if (refsW) {
      pvars.push(pluralVars.w);
    }
    if (refsF) {
      pvars.push(pluralVars.f);
    }
    if (refsT) {
      pvars.push(pluralVars.t);
    }

    if (!exactConditions.length && !keyConditions.length) {
      return other;
    }

    this.vars.s = null;
    this.vars.n = null;
    return types.sequenceExpression(vars.concat([exactConditions.reduceRight(function (alt, _ref) {
      var test = _ref.test;
      var expr = _ref.expr;

      return types.conditionalExpression(test, expr, alt);
    }, types.sequenceExpression(pvars.concat([keyConditions.reduceRight(function (alt, _ref2) {
      var test = _ref2.test;
      var expr = _ref2.expr;

      return types.conditionalExpression(test, expr, alt);
    }, other)])))]));
  };

  Transpiler.prototype.transpileSelect = function transpileSelect(id, children) {
    var _this5 = this;

    var s = types.identifier('s');
    var other = types.literal('');
    var conditions = [];
    _Object$keys(children).forEach(function (key) {
      if (key === 'other') {
        other = _this5.transpileSub(children[key]);
        return;
      }
      conditions.push({
        test: types.binaryExpression('===', s, types.literal(key)),
        expr: _this5.transpileSub(children[key])
      });
    });

    if (!conditions.length) {
      return other;
    }

    this.vars.s = null;
    return types.sequenceExpression([types.assignmentExpression('=', s, this.transpileArgument(id)), conditions.reduceRight(function (alt, _ref3) {
      var test = _ref3.test;
      var expr = _ref3.expr;

      return types.conditionalExpression(test, expr, alt);
    }, other)]);
  };

  Transpiler.prototype.transpileArgument = function transpileArgument(id) {
    return this.useArgExpressions ? this.useArgExpressions[id] : types.memberExpression(types.identifier('args'), types.literal(id), true);
  };

  Transpiler.transpile = function transpile(elements, options) {
    return new Transpiler(options).transpile(elements);
  };

  return Transpiler;
})();

exports['default'] = Transpiler;
module.exports = exports['default'];