'use strict';

var _classCallCheck = require('babel-runtime/helpers/class-call-check')['default'];

var _getIterator = require('babel-runtime/core-js/get-iterator')['default'];

var _interopRequireDefault = require('babel-runtime/helpers/interop-require-default')['default'];

var _interopRequireWildcard = require('babel-runtime/helpers/interop-require-wildcard')['default'];

exports.__esModule = true;

var _fs = require('fs');

var _path = require('path');

var _mkdirp = require('mkdirp');

var _mkdirp2 = _interopRequireDefault(_mkdirp);

var _babelCore = require('babel-core');

var babel = _interopRequireWildcard(_babelCore);

var _sourceMap = require('source-map');

var _sourceMap2 = _interopRequireDefault(_sourceMap);

var _chalk = require('chalk');

var _chalk2 = _interopRequireDefault(_chalk);

var _messageFormatParser = require('message-format/parser');

var _messageFormatParser2 = _interopRequireDefault(_messageFormatParser);

var _translateUtil = require('./translate-util');

var _transpiler = require('./transpiler');

var _transpiler2 = _interopRequireDefault(_transpiler);

var _packageJson = require('../package.json');

var _packageJson2 = _interopRequireDefault(_packageJson);

var types = babel.types;

/**
 * Base class for traversing source code to lint, extract, and inline messages
 **/

var Visitor = (function () {
  function Visitor() {
    var options = arguments[0] === undefined ? {} : arguments[0];

    _classCallCheck(this, Visitor);

    this.functionName = options.functionName || 'formatMessage';
    this.autoDetectFunctionName = 'autoDetectFunctionName' in options ? !!options.autoDetectFunctionName : true;
    this.getKey = _translateUtil.getGetKey(options);
    this.translate = _translateUtil.getTranslate(options);
    this.translations = options.translations;
    this.missingTranslation = options.missingTranslation;
    this.missingReplacement = options.missingReplacement;
    this.locale = options.locale || 'en';
    this.currentFileName = null;
    this.scopes = [];

    if (typeof options.emitWarning === 'function') {
      this.emitWarning = options.emitWarning;
    }
    if (typeof options.emitError === 'function') {
      this.emitError = options.emitError;
    }
    if (typeof options.emitResult === 'function') {
      this.emitResult = options.emitResult;
    }
    if (typeof options.emitFile === 'function') {
      this.emitFile = options.emitFile;
    }
  }

  Visitor.prototype.emitWarning = function emitWarning(message) {
    console.warn(message);
  };

  Visitor.prototype.emitError = function emitError(message) {
    console.error(message);
  };

  Visitor.prototype.emitResult = function emitResult(code) {
    console.log(code);
  };

  Visitor.prototype.emitFile = function emitFile(filename, content, sourcemap) {
    _mkdirp2['default'].sync(_path.dirname(filename));
    _fs.writeFileSync(filename, content, 'utf8');
  };

  Visitor.prototype.reportWarning = function reportWarning(node, message) {
    this.emitWarning(_chalk2['default'].bold.yellow(message) + '\n' + _chalk2['default'].yellow(this.getLocation(node)));
  };

  Visitor.prototype.reportError = function reportError(node, message, submessage) {
    this.emitError(_chalk2['default'].red(_chalk2['default'].bold(message) + '\n' + this.getLocation(node) + (!submessage ? '' : '\n    ' + submessage.replace(/\n/g, '\n    '))));
  };

  Visitor.prototype.getLocation = function getLocation(node) {
    var translate = this.isTranslateCall(node) ? '.translate' : '';
    return '    at ' + this.functionName + translate + ' (' + this.currentFileName + ':' + node.loc.start.line + ':' + node.loc.start.column + ')';
  };

  Visitor.prototype.isFormatImport = function isFormatImport(node) {
    return node && (types.isImportDeclaration(node) && types.isLiteral(node.source, { value: _packageJson2['default'].name }) && node.specifiers.length === 1 && types.isImportDefaultSpecifier(node.specifiers[0]));
  };

  Visitor.prototype.isFormatRequire = function isFormatRequire(node) {
    return node && (types.isCallExpression(node) && types.isIdentifier(node.callee, { name: 'require' }) && this.isString(node.arguments[0]) && this.getStringValue(node.arguments[0]) === _packageJson2['default'].name);
  };

  Visitor.prototype.isFormatCall = function isFormatCall(node) {
    return node && (types.isCallExpression(node) && types.isIdentifier(node.callee, { name: this.functionName }));
  };

  Visitor.prototype.isTranslateCall = function isTranslateCall(node) {
    var callee = node && node.callee;
    return callee && (types.isMemberExpression(callee) && types.isIdentifier(callee.object, { name: this.functionName }) && types.isIdentifier(callee.property, { name: 'translate' }));
  };

  Visitor.prototype.isReplaceable = function isReplaceable(node) {
    return node && (this.isFormatCall(node)
    // first argument is a literal string, or template literal with no expressions
     && this.isString(node.arguments[0]) && (
    // no specified locale, or is a literal string
    !node.arguments[2] || this.isString(node.arguments[2])) || this.isTranslateCall(node) && this.isString(node.arguments[0]) && (!node.arguments[1] || this.isString(node.arguments[1])));
  };

  Visitor.prototype.isString = function isString(node) {
    return node && (types.isLiteral(node) && typeof node.value === 'string' || types.isTemplateLiteral(node) && node.expressions.length === 0 && node.quasis.length === 1);
  };

  Visitor.prototype.getStringValue = function getStringValue(literal) {
    if (types.isTemplateLiteral(literal)) {
      return literal.quasis[0].value.cooked;
    }
    return literal.value;
  };

  Visitor.prototype.getPatternError = function getPatternError(pattern) {
    try {
      _messageFormatParser2['default'].parse(pattern);
    } catch (error) {
      return error;
    }
  };

  Visitor.prototype.forEachFile = function forEachFile(files, fn, ctx) {
    for (var _iterator = files, _isArray = Array.isArray(_iterator), _i = 0, _iterator = _isArray ? _iterator : _getIterator(_iterator);;) {
      var _ref;

      if (_isArray) {
        if (_i >= _iterator.length) break;
        _ref = _iterator[_i++];
      } else {
        _i = _iterator.next();
        if (_i.done) break;
        _ref = _i.value;
      }

      var file = _ref;

      var source = file;
      if (typeof file === 'string') {
        source = {
          sourceFileName: file,
          sourceCode: _fs.readFileSync(file, 'utf8')
        };
      }
      fn.call(ctx, source);
    }
  };

  Visitor.prototype.run = function run(_ref2) {
    var sourceCode = _ref2.sourceCode;
    var sourceFileName = _ref2.sourceFileName;
    var sourceMapName = _ref2.sourceMapName;
    var inputSourceMap = _ref2.inputSourceMap;

    var filename = this.currentFileName = sourceFileName;
    var shouldOutputCode = this.shouldOutputCode();
    try {
      return babel.transform(sourceCode, {
        // result properties to populate
        code: shouldOutputCode, // .code
        sourceMaps: shouldOutputCode, // .map
        ast: false, // .ast

        filename: filename,
        sourceMapName: sourceMapName,
        inputSourceMap: inputSourceMap,
        plugins: [new babel.Transformer('format-message', this.visitors())],
        whitelist: []
      });
    } catch (error) {
      if (!/^SyntaxError/.test(error.message) || !error.loc) {
        throw error;
      }
      var _error$loc = error.loc;
      var line = _error$loc.line;
      var column = _error$loc.column;

      this.emitError(_chalk2['default'].red(_chalk2['default'].bold('SyntaxError: Could not parse source code') + '\n' + '    at ' + sourceFileName + ':' + line + ':' + column + '\n' + '    ' + error.message));
      return shouldOutputCode ? { code: sourceCode, map: inputSourceMap } : {};
    }
  };

  Visitor.prototype.visitors = function visitors() {
    var _this = this;

    var self = this;
    // simplistic function and file scope tracking
    var _enter = function _enter() {
      _this.scopes.push(_this.functionName);
    };
    var _exit = function _exit() {
      _this.functionName = _this.scopes.pop();
    };
    return {
      Function: { enter: _enter, exit: _exit },

      // `this` is bound to the traverser, `self` is the visitor
      Program: {
        enter: function enter(node, parent, scope) {
          _enter();
          self.enterProgram(node, parent, scope, this);
        },
        exit: function exit(node, parent, scope) {
          _exit();
          self.exitProgram(node, parent, scope, this);
        }
      },

      VariableDeclarator: {
        exit: function exit(node) {
          if (self.autoDetectFunctionName && self.isFormatRequire(node.init)) {
            self.functionName = node.id.name;
          }
        }
      },

      ImportDeclaration: {
        exit: function exit(node) {
          if (self.autoDetectFunctionName && self.isFormatImport(node)) {
            self.functionName = (node.specifiers[0].local || node.specifiers[0].id).name;
          }
        }
      },

      CallExpression: {
        enter: function enter(node, parent, scope) {
          if (self.isFormatCall(node)) {
            self.enterFormatCall(node, parent, scope, this);
          } else if (self.isTranslateCall(node)) {
            self.enterTranslateCall(node, parent, scope, this);
          }
        },
        exit: function exit(node, parent, scope) {
          if (self.isFormatCall(node)) {
            self.exitFormatCall(node, parent, scope, this);
          } else if (self.isTranslateCall(node)) {
            self.exitTranslateCall(node, parent, scope, this);
          }
        }
      }
    };
  };

  // to be overridden

  Visitor.prototype.shouldOutputCode = function shouldOutputCode() {
    return false;
  };

  // to be overridden

  Visitor.prototype.enterFormatCall = function enterFormatCall() {};

  Visitor.prototype.exitFormatCall = function exitFormatCall() {};

  Visitor.prototype.enterTranslateCall = function enterTranslateCall() {};

  Visitor.prototype.exitTranslateCall = function exitTranslateCall() {};

  Visitor.prototype.enterProgram = function enterProgram() {};

  Visitor.prototype.exitProgram = function exitProgram() {};

  return Visitor;
})();

exports['default'] = Visitor;
module.exports = exports['default'];